
#include "mainwindow.h"
#include "kanaconfigdialog.h"
#include <QtGui>

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent){
	setupUi(this);
	
	curFile = "";
	curDataFile = ":/data.xml";
	printer = new QPrinter() ;
	
	actionCopy->setEnabled(false);
	action_Undo->setEnabled(false);
	action_Redo->setEnabled(false);
	
	readSettings();
	
	
	
	/** slots connections **/
	QObject::connect(actionAbout, SIGNAL(activated()), this, SLOT(about()) );
	QObject::connect(actionAbout_Qt, SIGNAL(activated()), this, SLOT(aboutQt()) );
	
	/** temporary slots connection (have to be removed from here to go to the ui_ file) **/
	QObject::connect(action_Undo, SIGNAL(activated()), kanaeditor, SLOT(editorUndo()) );
	QObject::connect(action_Redo, SIGNAL(activated()), kanaeditor, SLOT(editorRedo()) );
	QObject::connect(actionSave_as, SIGNAL(activated()), this, SLOT(saveAs()) );
	QObject::connect(action_Save, SIGNAL(activated()), this, SLOT(save()) );
	QObject::connect(action_Open, SIGNAL(activated()), this, SLOT(open()) );
	QObject::connect(action_New, SIGNAL(activated()), this, SLOT(fileNew()) );
	QObject::connect(action_Print, SIGNAL(activated()), this, SLOT(filePrint()) );
	QObject::connect(action_Settings, SIGNAL(activated()), this, SLOT(showSettings()) );
}

void MainWindow::loadData(const QString &file ){
	kanaeditor->setDataFile(file);
}

void MainWindow::showSettings() {
	maybeSave();
	writeSettings();
	KanaConfigDialog *d = new KanaConfigDialog(this);
	connect(d, SIGNAL(configurationUpdated()), this, SLOT(readSettings()) );
	d->exec();
}

void MainWindow::about()
{
	QMessageBox::about(this, tr("About PKanaEditor"), 
		tr("<h2>PKanaEditor 0.2</h2>"
		"<p>A small romanji to Kana text editor"
		"<br>"
		"<p>Author: Arnaud Dupuis <a.dupuis@infinityperl.org>"
		"<p>Copyright: Arnaud Dupuis - Infinity Perl"
		"<p>This software is released under the GNU/GPLv3 License."
		)
	);
}

void MainWindow::aboutQt()
{
	QMessageBox::aboutQt(this);
}

void MainWindow::fileNew(){
	kanaeditor->editorClear();
	setCurrentFile("");
}

void MainWindow::open()
{
	QString dir = QDir::homePath();
	if( !curFile.isEmpty() )
		dir = (QFileInfo(curFile).dir()).absolutePath();
	if (maybeSave()) {
		QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                            dir,
                            tr("Text (*.txt);;All (*.*)"));
		if (!fileName.isEmpty())
			loadFile(fileName);
	}
}

void MainWindow::loadFile(const QString &fileName)
{
	QFile file(fileName);
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
		QMessageBox::warning(this, tr("The \"P\" Kana Editor"),
					tr("Cannot read file %1:\n%2.")
					.arg(fileName)
					.arg(file.errorString()));
		return;
	}
	
	QTextStream in(&file);
// 	in.setAutoDetectUnicode(true);
	in.setCodec(QTextCodec::codecForName("UTF-8"));
	QApplication::setOverrideCursor(Qt::WaitCursor);
	kanaeditor->editorSetPlainText(in.readAll());
	QApplication::restoreOverrideCursor();
	
	setCurrentFile(fileName);
	statusBar()->showMessage(tr("File loaded"), 2000);
}

bool MainWindow::saveAs()
{
	QString dir = QDir::homePath();
	if( !curFile.isEmpty() )
		dir = (QFileInfo(curFile).dir()).absolutePath();
	QString fn = QFileDialog::getSaveFileName(this, tr("Save File"),
                            dir,
                            tr("Text (*.txt);;All (*.*)"));
	if (fn.isEmpty())
		return false;
	
	return saveFile(fn);
}

bool MainWindow::save()
{
	if (curFile.isEmpty()) {
		return saveAs();
	} else {
		return saveFile(curFile);
	}
}

bool MainWindow::saveFile(const QString &fileName)
 {
	QFile file(fileName);
	if (!file.open(QFile::WriteOnly | QFile::Text)) {
		QMessageBox::warning(this, tr("The \"P\" Kana Editor"),
				tr("Cannot write file %1:\n%2.")
				.arg(fileName)
				.arg(file.errorString()));
		return false;
	}
	
	QTextStream out(&file);
// 	out.setGenerateByteOrderMark(true);
	out.setCodec(QTextCodec::codecForName("UTF-8"));
	
	QApplication::setOverrideCursor(Qt::WaitCursor);
	out << kanaeditor->editorToPlainText();
	QApplication::restoreOverrideCursor();
	
	setCurrentFile(fileName);
	statusBar()->showMessage(tr("File saved"), 2000);
	return true;
}

void MainWindow::setCurrentFile(const QString &fileName)
{
	qDebug() << "setCurrentFile(" << fileName <<")";
	curFile = fileName;
	kanaeditor->editorDocument()->setModified(false);
	setWindowModified(false);
	
	QString shownName;
	if (curFile.isEmpty())
		shownName = "untitled.txt";
	else
		shownName = strippedName(curFile);
	
	qDebug() << "shownName: " << shownName ;
	
	setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(tr("The \"P\" Kana Editor")));
}

QString MainWindow::strippedName(const QString &fullFileName)
{
	return QFileInfo(fullFileName).fileName();
}

void MainWindow::writeSettings()
{
	QSettings settings("InfinityPerl", "PKanaEditor");
	settings.setValue("pos", pos());
	settings.setValue("size", size());
	settings.setValue("fileName", curFile);
	settings.setValue("dataFile",curDataFile);
}

void MainWindow::readSettings()
{
	QSettings settings("InfinityPerl", "PKanaEditor");
	QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
	QSize size = settings.value("size", QSize(800, 600)).toSize();
	QString file = settings.value("fileName", "").toString() ;
	curDataFile = settings.value("dataFile",":/data.xml").toString();
	if( !file.isEmpty() )
		loadFile(file);
	else
		setCurrentFile("");
	resize(size);
	move(pos);
	loadData(curDataFile);
}

void MainWindow::closeEvent(QCloseEvent *event)
{
	if (maybeSave()) {
		writeSettings();
		event->accept();
	} else {
		event->ignore();
	}
}
bool MainWindow::maybeSave()
{
	if (kanaeditor->editorDocument()->isModified()) {
		QMessageBox::StandardButton ret;
		ret = QMessageBox::warning(this, tr("The \"P\" Kana Editor"),
			tr("The document has been modified.\n"
				"Do you want to save your changes?"),
			QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
		if (ret == QMessageBox::Save)
		return save();
		else if (ret == QMessageBox::Cancel)
		return false;
	}
	return true;
}

void MainWindow::filePrint() {
	QPrintDialog printDialog(printer, this);
	qDebug() << "printer name is : " << printer->printerName() ;
	if( printer->printerName().isEmpty() ){
		qDebug() << "printerName is empty !!";
		printer->setOutputFormat( QPrinter::PdfFormat );
		printer->setOutputFileName( curFile + ".pdf" );
	}
	if( printDialog.exec() ){
		qDebug() << "printer is : " << printer ;
		QPainter painter(printer);
		qDebug() << "painter is : " << &painter;
// 		QTextDocument textDocument;
// 		textDocument.setHtml( (kanaeditor->editorDocument())->toHtml( "utf-8" ) );
// 		textDocument.print(printer);
		
		kanaeditor->editorDocument()->print(printer);
	}
}
